<?php
require_once('object_class.php');

/**
 * A host definition is used to define a physical server, workstation, device,
 * etc. that resides on your network.
 */
class host_class extends object_class
{
	/** The nagios object type we're working with */
	var $type = 'host';
	/** Default values that Nagios assumes if no values are set */
	var $nagios_defaults =
		array('stalking_options' => array('n' => ''),
		      'notification_options' => array('n' => ''),
		      'flap_detection_options' => array('o' => 1, 'd' => 1, 'u' => 1),
		      'obsess' => '1',
		      'check_freshness' => '0',
		      );
	var $slave_types = array(
		'hostextinfo',
		'hostescalation',
		'hostdependency',
		'service');

	function save_object($user=true)
	{
		$ret = parent::save_object($user);
		if (!$ret)
			return $ret;

		$dep_objs = $this->find_slave_objects('hostextinfo');
		if (isset($dep_objs)) {
			foreach ($dep_objs as $extinfo_id) {
				$extinfo = new hostextinfo_class($extinfo_id);
				$extinfo->banish_object();
			}
		}

		return $ret;
	}

	function validate_object() {
		if (!empty($this->obj['host_name']) && empty($this->obj['address']))
			$this->obj['address'] = $this->obj['host_name'];

		return parent::validate_object();
	}

	public function get_cmd_bar_links()
	{
		$cmd_links = parent::get_cmd_bar_links();
		// instead of duplicating validation from parent, just check if the validation ended up allowing copying, and only then add clone
		if (isset($cmd_links['copy']))
			$cmd_links['clone'] = '<a title="Clone host" href="clone.php?obj_type=host&amp;obj_id='.$this->id.'">';
		return $cmd_links;
	}
}
