<?php
/**
 * HTTP API entry point
 */
class http_api_class
{

	private function index_html_response($body = null, $status_code = 200, $title = null, $long_message = null)
	{
		if($title) {
			$title = " - ".$title;
		}
		$title = "op5 Monitor API".$title;
		$help_class = new help_class();
		return new Api_Response(sprintf("<!DOCTYPE html>
<html><head>
<link type='text/css' rel='stylesheet' href='/api/css/monitor.css' />
<title>%s</title>
</head><body>
<div id='container'>
	<div id='sidebar'>
		<div id='monitor'>op5 Monitor</div>
		<p>HTTP API</p>
		<ul>
			<li class='current'>Config</li>
			<li><a href='/api/help/status'>Status</a></li>
		</ul>
		<p>This is the op5 Monitor configuration API. Each Nagios object can be listed, created, updated or deleted by using the following documentation.</p>

		<p>Each object type is bundled with examples for you to copy and paste, to get you going.</p>
		<h2>Object types</h2>
		%s

		<h2>Troubleshooting</h2>
		<ul>
			<li><a href='#trouble_ssl'>\"SSL certificate is invalid\"</a></li>
			<li><a href='#trouble_connect'>\"Can I connect to the API in another way?\"</a></li>
			<li><a href='#trouble_method'>\"What's with PUT, DELETE, GET..?\"</a></li>
			<li><a href='#trouble_status_code'>\"Why am I getting a bad status code, like 400?\"</a></li>
			<li><a href='#trouble_custom_variables'>\"How do I handle custom variables?\"</a></li>
			<li><a href='#trouble_resource'>\"What is a 'resource'?\"</a></li>
			<li><a href='#trouble_unknown'>\"I am experiencing problems, where should I go next?\"</a></li>
		</ul>
	</div>
	<div id='main'>
		%s

		<h2>Using the <a href='http://en.wikipedia.org/wiki/Representational_state_transfer#RESTful_web_services' title='Representational state transfer'>REST</a> API</h2>

		<h3>Introduction</h3>

		<p>This is the REST-API that let's you configure op5 Monitor by issuing regular HTTP requests. Basically, you 'visit' an URI, which triggers Monitor to do something, and you get a response telling you what happened.</p>

		<p>You get help with the formulation of the requests on each object type's help page (look over to your left to find which object types are available). <strong>You also get tips on how to perform HTTP requests via the terminal (using cURL) and via scripting languages (using Python or PHP).</strong></p>

		<p>A final, important, note on using this configuration API: <strong>no changes are inserted until you manually confirm them</strong>, <a href='#persistent_changes'>instructions on saving changes are found below</a>.</p>

		<h3>Technical information</h3>

		<h4>Authentication</h4>

		<p>You send along a pair of regular op5 Monitor user credentials in a basic auth header with every request. View any example and you will understand how to do this.</p>

		<h4>Requests</h4>
		<p>In a REST-API, there are a couple of important things that make up for a valid request:</p>
		<ul>
			<li>
				The URI, in the following form
				<ul>
				<li>/api/config/host</li>
				<li>/api/config/host/a-hostname</li>
				</ul>
			</li>
			<li>
				The HTTP method
				<ul>
					<li>GET: lists objects.</li>
					<li>POST: creates objects.</li>
					<li>PUT: overwrite objects.</li>
					<li>PATCH: update objects.</li>
					<li>DELETE: delete objects.</li>
				</ul>
			</li>
			<li>
				The format, appended to the URI
				<ul>
					<li>?format=xml</li>
					<li>?format=json (default, except for requests from a browser)</li>
					<li>?format=html (default in a browser environment)</li>
				</ul>
			</li>
		</ul>

		<p>Once again, viewing any of the examples helps you get into the REST mindset.</p>

		<h4>Responses</h4>
		<p>You need to examine two things in the response: the status code and the body. The status code tells you what Monitor did with your input and the body gives you a detailed response.</p>

		<p>Examples of status codes:</p>
		<ul>
			<li>200: Everything went fine.</li>
			<li>201: Everything went fine, you wanted to create an object and it was created.</li>
			<li>400: You sent a request Monitor didn't understand. Correct the input and try again.</li>
			<li>401: Authentication required.</li>
			<li>403: You're not authorized to do this.</li>
			<li>404: The URI doesn't exist.</li>
		</ul>

		<p>These status codes are general for all web traffic and can be found at the <a href='http://en.wikipedia.org/wiki/HTTP_status_code'>List of HTTP status codes</a></p>

		<h4 id='persistent_changes'>Making changes persistent</h4>
		<p>Just as in the web GUI of op5 Monitor, when you issue changes they are not immidiately inserted into the Nagios configuration files. An additional action is required and here is how you do it via the HTTP API:</p>
		<ul>
			<li>Perform a <strong>GET</strong> request to /api/config/change to see all the changes you have made, which are to be committed.
			<ul>
				<li>If you wish to throw away the changes: perform a <strong>DELETE</strong> request to /api/config/change, which undos all changes.</li>
				<li>If you are satisfied with the changes: perform a <strong>POST</strong> request to /api/config/change.</li>
			</ul>
			</li>
		</ul>

		<h3>Troubleshooting</h3>
		<dl>
			<dt id='trouble_ssl'>\"SSL certificate is invalid\"</dt>
			<dd>
			<p>By default, op5 Monitor ships with an unsigned SSL certificate. If you do not replace this with a signed certificate, the most HTTP access methods (including the ones given in the examples) will deny your access. This is solved the following way:</p>

			<p>cURL: Add a <code>--insecure</code> flag:</p>
			<div class='curl'>
				<pre><code>curl &lt;API endpoint here&gt; <strong>--insecure</strong> -u &lt;your username&gt;:&lt;your password&gt;</code></pre>
			</div>

			<p>Python: Adding a <code>verify=False</code> flag:</p>
			<div class='python'>
				<pre><code>requests.get('&lt;API endpoint here&gt;', auth=('&lt;your username&gt;', '&lt;your username&gt;'), <strong>verify=False</strong>)</code></pre>
			</div>

			<p>PHP with cURL: Adding a <code>CURLOPT_SSL_VERIFYPEER</code> flag:</p>
			<div class='php'>
				<pre><code>curl_setopt(\$a_handle, <strong>CURLOPT_SSL_VERIFYPEER</strong>, false);</code></pre>
			</div>
			</dd>

			<dt id='trouble_connect'>\"Can I connect to the API in another way?\"</dt>
			<dd>Yes, all you need is a way to perform a HTTP method in your language. Study the minimal examples and take note of which HTTP method is used, which URI that's used and what data is sent to the URI.</dd>

			<dt id='trouble_method'>\"What's with PUT, DELETE, GET..?\"</dt>
			<dd>These are HTTP methods. There are examples of each method on the object's page.</dd>

			<dt id='trouble_status_code'>\"Why am I getting a bad status code, like 400?\"</dt>
			<dd>The HTTP response status codes are quite general and Monitor tries to follow the formal descriptions in every possible situation. Therefore, Wikipedia's article <a href='http://en.wikipedia.org/wiki/HTTP_status_code'>List of HTTP status codes</a> should help you get back on track; for example, it says that the code 400 indicates bad syntax of the request.</dd>

			<dt id='trouble_custom_variables'>\"How do I handle custom variables?\"</dt>
			<dd>Custom variables are just key-values stored with each Nagios object, as such, the API treats them the same way. Just make sure that each key is prepended with an underscore (_), otherwise the request will be returned with an erroneous status code.</dd>

			<dt id='trouble_resource'>\"What is a 'resource'?\"</dt>
			<dd><p>When listing objects, you are given an additional property called <em>resource</em>. This let's you know the resource's location to which you might send a POST or PUT request. Do note that the resource is not permanent, since a removed/renamed object will make the URI cease to exist.</p>

			<p>The resource is urlencoded so that it is always usable in its current form.</p></dd>

			<dt id='trouble_unknown'>\"I am experiencing problems, where should I go next?\"</dt>
			<dd>You could contact your op5 Monitor technician or visit <a href='http://www.op5.com/support/'>http://www.op5.com/support/</a> to acquire more help.</dd>
		</dl>
	</div>
</div>
</body></html>",
			$title,
			$help_class->list_objects('/api/help/config/'), // @todo adapt to full URI
			$body),
		$status_code,
		$long_message);
	}

	private function html_response($body, $status_code = 200, $title = null, $long_message = null)
	{
		if($title) {
			$title = " - ".$title;
		}
		$title = "op5 Monitor API".$title;
		$help_class = new help_class();
		return new Api_Response(sprintf("<!DOCTYPE html>
<html><head>
<link type='text/css' rel='stylesheet' href='/api/css/monitor.css' />
<title>%s</title>
</head><body>
<div id='container'>
 <div id='sidebar'>
         <div id='monitor'>op5 Monitor</div>
         <p>HTTP API</p>
	 <ul>
		<li class='current'><a href='/api/help/config'>Config</a></li>
		<li><a href='/api/help/status'>Status</a></li>
	 </ul>
         <p>This is the op5 Monitor configuration API. Each Nagios object can be listed, created, updated or deleted by using the following documentation.</p>

         <p>Each object type is bundled with examples for you to copy and paste, to get you going.</p>
         <h2>Object types</h2>
         %s
 </div>
 <div id='main'>
         %s
 </div>
</div>
</body></html>",
			$title,
			$help_class->list_objects(),
			$body),
		$status_code,
		$long_message);
	}

	/**
	 * Returns a properly formatted view of the object,
	 * expanded, without internal ID numbers, and with sub-objects
	 * included.
	 *
	 * @param $obj object_class
	 * @return array
	 */
	private function output_object(object_class $obj)
	{
		$obj->expand_object();
		$obj->filter_relations();
		$res = $obj->denormalize();
		$prop = $obj->get_properties();
		if (isset($prop['deprecated_names'])) {
			foreach ($prop['deprecated_names'] as $old => $new) {
				if (isset($res[$new]))
					$res[$old] = $res[$new];
			}
		}
		unset($res['id']);
		foreach ($obj->find_slave_objects() as $type => $ary) {
			$res[$type.'s'] = array();
			foreach ($ary as $subobj) {
				$class = $type.'_class';
				$subobj = new $class($subobj);
				$subres = $this->output_object($subobj);
				unset($subres[$subobj->master_obj_identifier]);
				$res[$type.'s'][] = $subres;
			}
		}
		return $res;
	}

	/**
	 * Generate a map of objects from a posted info-map
	 *
	 * We have the following nesting to support today:
	 * host =>
	 *     hostdependency,
	 *     hostescalation,
	 *     service =>
	 *         servicedependency,
	 *         serviceescalation
	 */
	private function generate_objmap(object_class $obj, $info)
	{
		// Used for fake parent relations, this will be un-nested in save-objects
		// The idea is copied from host_wizard
		// Fun fact: $obj->obj['id'] may not be negative or validation fails.
		// $obj->id may, and everything breaks if it isn't set to something.
		static $next_id = -1;
		if ($obj->id === false || $obj->id === 'new')
			$obj->id = $next_id--;
		$all_referenced = array();
		if (is_array($obj->slave_types)) {
			foreach ($obj->slave_types as $type) {
				if (!isset($info[$type.'s']))
					continue;
				$all_referenced[$type] = array();
				if (!is_array($info[$type.'s'])) {
					throw new Api_Error_Response('Invalid request', 400, "Key '{$type}s' must contain a list");
				}
				foreach ($info[$type.'s'] as $subinfo) {
					if (!is_array($subinfo)) {
						throw new Api_Error_Response('Invalid request', 400, "Key '{$type}s' must contain a list");
					}
					$class = $type.'_class';
					$subobj = new $class();
					$subobj->get_default_object();
					$subobj->set_master_object_type($obj->type);
					$subobj->set_master_object_id($obj->id);
					if ($type == 'service') {
						$subobj->get_object_by_name($info[$obj->properties['topic']].';'.$subinfo['service_description']);
					}
					else if ($type == 'hostdependency') {
						$hostgroup = 0;
						$host = 0;
						if (isset($subinfo['host_name']))
							$host = get_object_index_id('host', $subinfo['host_name']);
						if (isset($subinfo['hostgroup_name']))
							$hostgroup = get_object_index_id('hostgroup', $subinfo['hostgroup_name']);
						$query = "select id from $type where {$subobj->master_obj_identifier} = {$obj->id} and host_name = $host and hostgroup_name = $hostgroup";
						$res = pdo_exec_query($query);
						$row = $res->fetch(PDO::FETCH_NUM);
						unset($res);
						if ($row) {
							$subobj->id = $row[0];
							$subobj->get_object();
						}
						unset($res);
					}
					else if ($type == 'servicedependency') {
						$service = get_object_index_id('service', $subinfo['service']);
						$query = "select id from $type where {$subobj->master_obj_identifier} = {$obj->id} and service = $service";
						$res = pdo_exec_query($query);
						$row = $res->fetch(PDO::FETCH_NUM);
						unset($res);
						if ($row) {
							$subobj->id = $row[0];
							$subobj->get_object();
						}
					}
					$sub_referenced = $this->generate_objmap($subobj, $subinfo);
					$all_referenced[$type][$sub_referenced['object']->id] = $sub_referenced;
				}
				unset($info[$type.'s']);
			}
		}
		$obj->normalize($info, true);
		$all_referenced['object'] = $obj;

		return $all_referenced;
	}

	/**
	 * We have arbitrarily decided that PUTing a sub-object element
	 * to an object means we need to delete any preexisting sub-objects
	 * of that type. That is, PUTing the host
	 * {'host_name':'foo','services':[],'hostdependencys':[{'host_name':'monitor'}]}
	 * means we delete any services attached to the object, we delete all
	 * dependencies and create a new one, but we do not touch hostescalations.
	 */
	private function purge_other_subs($objs)
	{
		$obj = $objs['object'];
		foreach ($obj->find_slave_objects() as $type => $ary) {
			$class = $type.'_class';
			if (isset($objs[$type])) {
				foreach ($ary as $obj_id) {
					if (isset($objs[$type][$obj_id])) {
						/* this is a service, clean service sub objs */
						$this->purge_other_subs($objs[$type][$obj_id]);
						continue;
					}
					$obj = new $class($obj_id);
					if (!$obj->banish_object()) {
						return array($obj->get_object_name() => $obj->delete_message);
					}
				}
			}
		}
		return true;
	}

	/**
	 * Baring bugs, this should detect any and all problems with all the listed
	 * objects, so we can bail out before we do any changes.
	 *
	 * @return true when validation was successful, array otherwise
	 */
	private function validate_objects($objs)
	{
		$obj = $objs['object'];
		unset($objs['object']);
		if (!$obj->validate_object()) {
			return array($obj->get_object_name() => $obj->validation_error_list);
		}
		if (empty($objs))
			return true;

		foreach ($objs as $type => $subobjs) {
			foreach ($subobjs as $subobj) {
				if (($res = $this->validate_objects($subobj)) !== true)
					return $res;
			}
		}
		return true;
	}

	/**
	 * Validate that all objects in $objs can be saved, and then save them.
	 */
	private function save_objects($objs)
	{
		$obj = $objs['object'];
		unset($objs['object']);
		if ($obj->id < 0)
			$obj->id = false;
		if (!$obj->save_object()) {
			/* this shouldn't happen, but who knows? */
			return array($obj->get_object_name() => $obj->validation_error_list);
		}
		if (empty($objs))
			return true;

		foreach ($objs as $type => $subobjs) {
			foreach ($subobjs as $subobj) {
				$subobj['object']->set_master_object_id($obj->id);
				if (($res = $this->save_objects($subobj)) !== true)
					return $res;
			}
		}
		return true;
	}

	/**
	 * This will be executed for calls to the /change/ pseudo-type
	 *
	 * This allows users to GET pending changes, POST them to disk, and DELETE
	 * them all to restore a previous state.
	 */
	protected function handle_change(Api_Request $request)
	{
		if (!in_array(NACOMA_ACCESS_FULL_ACCESS, get_nacoma_access('export')))
			return new Api_Error_Response("You're not authorized to save config changes", 403);

		switch($_SERVER['REQUEST_METHOD']) {
		 case 'GET':
			$cl = new Changelog();
			return new Api_Response($cl->get_list());
		 case 'POST':
			$cl = new Changelog();
			if (($err = do_export($cl)) === true) {
				switch(export_reload_monitor()) {
				 case 1:
					return new Api_Error_Response("Monitor doesn't appear to be running", 500, "This could indicate a more serious error, so restarting not attempted.");
				 case 2:
					return new Api_Error_Response("Failed to open monitor command pipe for writing", 500, "This can happen when Monitor isn't running.");
				 default:
					return new Api_Response("Changes saved");
				}
			} else {
				return new Api_Error_Response('Export failed', 500, $err);
			}
		 case 'DELETE':
			revert_users_changes();
			return new Api_Response("Changes reverted");
		 default:
			return new Api_Error_Response('Invalid method "'.$_SERVER['REQUEST_METHOD'].'"', 405, "Only GET, POST and DELETE requests are allowed when handling changes");
		}
	}

	/**
	 * This will be executed for help calls
	 */
	protected function handle_help(Api_Request $request, $type)
	{
		if (!$_SERVER['REQUEST_METHOD'] === 'GET')
			return Api_Error_Response::with_usage('Invalid request', 400, $request);
		$help_class = new help_class();
		if (!$type)
			return $this->index_html_response();
		else if (array_key_exists($type, $help_class->get_available_objects()))
			return $this->html_response($help_class->get_html_help_for_object_type($type), 200, ucfirst(str_replace('_', ' ', $type)));
		else
			return Api_Error_Response::with_usage("Invalid object type given: '$type'", 404, $request);
	}

	/**
	 * This will be executed for "regular" API requests that show/create/modify
	 * nacoma objects
	 */
	protected function handle_regular(Api_Request $request, $type, $name=false)
	{
		if (($owner = get_lock_owner('all', 0))) {
			return new Api_Error_Response("The database is currently being exported by $owner",500, "This should merely take a few seconds, but while it's going on, no other users may edit the database.");
		}

		$class = $type . '_class';
		$obj = new $class();
		$params = $request->get_params();

		if (isset($params['parent_type']))
			$obj->set_master_object_type($params['parent_type']);
		if ($name)
			$obj->get_object_by_name($name);

		if ($name === false) {
			switch ($_SERVER['REQUEST_METHOD']) {
			 case 'GET':
				$objs = $obj->get_objects_for_user();
				$out = array();
				if (is_array($objs)) {
					foreach ($objs as $obj) {
						$out[] = array('name' => $obj, 'resource' => rtrim($request->get_full_url(), '/').'/'.urlencode($obj));
					}
				}
				return new Api_Response($out);
			 case 'POST':
				if (!$obj->can_user_write($obj->type, 'new'))
					return new Api_Error_Response('Permission Denied', 403);
				try {
					$body = $request->get_body();
				} catch(Exception $e) {
					return new Api_Error_Response('Bad Request', 400, $e->getMessage());
				}
				try {
					$obj->get_default_object();
					$objs = $this->generate_objmap($obj, $body);
					$existing = new $class($objs['object']->get_object_name());
					unset($objs['object']->obj['id']);
					if ($existing->id !== false && $existing->id !== 'new')
						return new Api_Error_Response('Object already exists', 409);
					if (($errs = $this->validate_objects($objs)) !== true)
						return new Api_Error_Response("Couldn't save object", 400, $errs);
					if (($errs = $this->save_objects($objs)) !== true)
						return new Api_Error_Response("Couldn't save object", 400, $errs);
					return new Api_Response($this->output_object($obj), 201);
				} catch (NacomaRelationException $ex) {
					return new Api_Error_Response("Couldn't save object", 400, $ex->getMessage());
				}
			 default:
				return Api_Error_Response::with_usage('Invalid method "'.$_SERVER['REQUEST_METHOD'].'"', 405, $request);
			}
		}

		switch ($_SERVER['REQUEST_METHOD']) {
		 case 'GET':
			if (!$obj->can_user_read($obj->type, $obj->id))
				return new Api_Error_Response('Permission Denied', 403);
			if (!$obj->id || $obj->id === 'new')
				return new Api_Error_Response('Object not found', 404);
			return new Api_Response($this->output_object($obj));
		 case 'PATCH':
			if (!$obj->id || $obj->id === 'new')
				return new Api_Error_Response('Object not found', 404);
			if (!$obj->can_user_write($obj->type, $obj->id))
				return new Api_Error_Response('Permission Denied', 403);
			try {
				$body = $request->get_body();
			} catch(Exception $e) {
				return new Api_Error_Response('Bad Request', 400, $e->getMessage());
			}
			try {
				$objs = $this->generate_objmap($obj, $body);
				if (count($objs) > 1)
					return new Api_Error_Response("You may not PATCH sub-objects", 501);
				if (($errs = $this->validate_objects($objs)) !== true)
					return new Api_Error_Response("Couldn't save object", 400, $errs);
				if (($errs = $this->save_objects($objs)) !== true)
					return new Api_Error_Response("Couldn't save object", 400, $errs);
				return new Api_Response($this->output_object($obj), 200);
			} catch (NacomaRelationException $ex) {
				return new Api_Error_Response("Couldn't save object", 400, $ex->getMessage());
			}
		 case 'PUT':
			if (!$obj->can_user_write($obj->type, $obj->id))
				return new Api_Error_Response('Permission Denied', 403);
			try {
				$body = $request->get_body();
			} catch(Exception $e) {
				return new Api_Error_Response('Bad Request', 400, $e->getMessage());
			}
			try {
				$obj->get_default_object();
				$objs = $this->generate_objmap($obj, $body);
				if (($errs = $this->validate_objects($objs)) !== true)
					return new Api_Error_Response("Couldn't save object", 400, $errs);
				if (($errs = $this->purge_other_subs($objs)) !== true)
					return new Api_Error_Response("Couldn't save object", 400, $errs);
				if (($errs = $this->save_objects($objs)) !== true)
					return new Api_Error_Response("Couldn't save object", 400, $errs);
				return new Api_Response($this->output_object($obj), 201);
			} catch (NacomaRelationException $ex) {
				return new Api_Error_Response("Couldn't save object", 400, $ex->getMessage());
			}
		 case 'DELETE':
			if (!$obj->can_user_write($obj->type, $obj->id))
				return new Api_Error_Response('Permission Denied', 403);
			if (!$obj->id || $obj->id === 'new')
				return new Api_Error_Response('Object not found', 404);
			if ($obj->banish_object())
				 return new Api_Response("$type $name deleted");
			else
				 return new Api_Error_Response("Couldn't delete $type $name", 400, $obj->delete_message);
		 default:
			return Api_Error_Response::with_usage('Invalid method "'.$_SERVER['REQUEST_METHOD'].'"', 405, $request);
		}
	}

	/**
	 * Main worker for this class, takes a request and returns a response
	 */
	public function handle_request(Api_Request $request)
	{
		if (!get_last_import_time() || changes_since_last_import()) {
			import_config($request->get_username());
		}

		$path = $request->get_path();
		$type = false;
		if (!empty($path))
			$type = array_shift($path);

		if ($request->is_help()) {
			return $this->handle_help($request, $type);
		}
		else if (!$type) {
			return Api_Error_Response::with_usage('Missing object type', 400, $request);
		}

		$class = $type . '_class';
		$name = false;
		if (!empty($path))
			$name = array_shift($path);

		if (!empty($path))
			return Api_Error_Response::with_usage('Invalid request', 404, $request);

		if ($type === 'change') {
			return $this->handle_change($request, $type);
		} else if (!in_array($class, help_class::$blacklist) && class_exists($class) && is_subclass_of($class, 'object_class')) {
			return $this->handle_regular($request, $type, $name);
		} else if ($type == 'role') {
			return new Api_Error_Response("Role objects no longer exists", 404);
		} else {
			return Api_Error_Response::with_usage('Invalid object type "'.$type.'"', 404, $request);
		}
	}
}
