<?php
/*
 * Copyright(C) 2003-2005, 2007 op5 AB
 * All rights reserved.
 *
 */

require_once('include/import.inc.php');
require_once('include/webconfig.inc.php');
require_once('include/check_mk.inc.php');
require_once('include/object_properties.inc.php');
require_once('include/objlock.inc.php');
require_once('classes/common_gui.php');
require_once('op5/log.php');

$access = get_nacoma_access('configuration_information');
if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
	$page->html_start("Access denied!");
	echo "<p class=\"note\">It appears as though you aren't authorized to access the configuration interface.</p>
			<p>Note that at least one of your groups must be authorized for both <strong>'configuration_information'</strong>
			and <strong>'system_commands'</strong> to be able change any configuration.</p>";
}
else {
	$page->html_start('Configure');

	die_if_locked_hard();

	$rights = get_rights();
	# import or not?
	if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'Import') {
		echo '<div class="note">';
		$rights = get_rights();
		if ((isset($rights['configuration_all'])) && isset($_REQUEST['force'])) {
			$page->run_import('Import forced');
		} else {
			revert_users_changes();
			if (isset($rights['configuration_all']))
				print '<p>You might want to do a <a href="configure.php?action=Import&force=force">complete reimport</a>?</p>';
		}
		$page->js_remove_save_warning();
		echo '</div>';
		check_mk_config_cleanup();
	}

	$page->import_if_changes();

	echo '<p>This is the op5 Monitor Configuration tool - the place where you add new hosts,
	change notification settings, handle contacts and so on. Basically just about
	every configuration related setting is managed from this tool.</p>';

	echo "<p><strong>To get started, select the prefered action below.</strong></p>";

	// this must be done AFTER importing.
	get_object_index_table('host');

	// Top level info text goes here.
	echo "<ul class=\"config\">\n";

	$Menu_Item = array();
	if(is_super_user() && object_class::can_user_write('host', 'new') && object_class::can_user_write('service', 'new')) {
		echo '<li><img src="images/icons/new-host.png" alt="" style="display: inline" /> <a title="Scan for new hosts and functions" href="host_wizard.php">New hosts</a></li>'."\n";
		$Menu_Item['autoscan.php'] = 'Network Autoscan';
	}

	$access = array_merge(get_nacoma_access('host_template'), get_nacoma_access('service_template'), get_nacoma_access('contact_template'));
	if (in_array(NACOMA_ACCESS_VIEW_ALL, $access))
		$Menu_Item['edit.php?obj_type=template'] = 'Templates';
	$access = get_nacoma_access('hostgroup');
	if (in_array(NACOMA_ACCESS_VIEW_ALL, $access) || in_array(NACOMA_ACCESS_VIEW_CONTACT, $access))
		$Menu_Item['edit.php?obj_type=hostgroup'] = 'Host Groups';
	$access = get_nacoma_access('servicegroup');
	if (in_array(NACOMA_ACCESS_VIEW_ALL, $access) || in_array(NACOMA_ACCESS_VIEW_CONTACT, $access))
		$Menu_Item['edit.php?obj_type=servicegroup'] = 'Service Groups';
	$access = get_nacoma_access('contact');
	if (in_array(NACOMA_ACCESS_VIEW_ALL, $access))
		$Menu_Item['edit.php?obj_type=contact'] = 'Contacts';
	$access = get_nacoma_access('contactgroup');
	if (in_array(NACOMA_ACCESS_VIEW_ALL, $access))
		$Menu_Item['edit.php?obj_type=contactgroup'] = 'Contact Groups';
	$access = get_nacoma_access('command');
	if (in_array(NACOMA_ACCESS_VIEW_ALL, $access))
		$Menu_Item['edit.php?obj_type=command'] = 'Commands';
	$Menu_Item['prototype.php'] = 'Profiles';
	$access = get_nacoma_access('timeperiod');
	if (in_array(NACOMA_ACCESS_VIEW_ALL, $access))
		$Menu_Item['edit.php?obj_type=timeperiod'] = 'Time Periods';

	$access = get_nacoma_access('nagvis');
	if (in_array(NACOMA_ACCESS_ADD_DELETE, $access)) {
		$Menu_Item['nagvisls.php'] = 'Nagvis permissions';
	}


	$access = get_nacoma_access('access_rights');
	if (in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
		// Local Users are only relevant if a local auth driver is configured.
		$userdrivers = $op5_auth->getAuthModulesPerDriver( 'Default' );
		if( false !== $userdrivers ) {
			$Menu_Item['edit_special.php/access'] = 'Local Users';
		}
		$Menu_Item['authconfig.php'] = 'Auth Modules';
		$Menu_Item['edit_special.php/group'] = 'Assign group rights';
	}


	$access = get_nacoma_access('pnp');
	if (in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
		$Menu_Item['pnp.php'] = 'Combined Graphs';
		$Menu_Item['edit_special.php/pnp_pages'] = 'Graph Collections';
	}

	foreach($Menu_Item as $page_link => $label) {
		echo '<li><img src="images/icons/'.(str_replace(' ','-',strtolower($label))).'.png" alt="" style="display: inline" /> <a href="'.$page_link.'" title="'.$label.' Configuration">'.$label.'</a></li>'."\n";
	}

	# conditionally add link to export to statistics
	if(is_dir('/opt/statistics')) {
		echo '<li><img src="images/icons/export-hosts-to-statistcs.png" alt="" /> <a href="statistics_export.php">Export hosts to statistics</a></li>'."\n";
	}

	echo '<li class="wide">';
				$page->form_start('host_form', 'get', 'edit.php');
	echo '<div>
				<input type="hidden" name="obj_type" value="host" />
				<img src="images/icons/host.png" alt="" /> Edit Host &nbsp;';
				$page->make_filtered_selection(get_objects_for_user('host'), '', 'obj_id');
				$page->submit_button('Go', 'action', true);
	echo "</form></div></li>\n";

	echo "</ul>\n";

	?>
	<p style="clear:both; padding-top: 30px"><strong>Tip:</strong> If you need help you can always click on the
	<img src="images/icons/square-help.png" alt="" style="margin-bottom: -3px" />
	in the upper right corner, you can also click on the
	<img src="images/icons/shield-help.png" alt="" style="margin-bottom: -2px" /> for each configurable variable.</p>

	<?php
}
$page->html_end();
