<?php
/*
 * Copyright(C) 2003-2007 op5 AB
 * All rights reserved.
 *
 */

require_once('config.inc.php');
require_once('system.inc.php');
require_once('data.inc.php');
require_once('op5/log.php');

/****************************************************************************/
/*************************** ACTION FUNCTIONS *******************************/
/****************************************************************************/

/* makes a comma-separated string of $arr's association keys */
function stringify_checkbox($arr) {
	$str = '';

	if(!is_array($arr)) return($arr);

	/* handle 'n'(for never) option for various variables */
	if(isset($arr['n'])) {
		if(count($arr) === 1)
			return('n');
		else
			unset($arr['n']);
	}
	$interesting = array_keys($arr);
	sort($interesting);

	$str = implode(',', $interesting);

	return($str);
}

/****************************************************************************/
/**************************** MISC FUNCTIONS ********************************/
/****************************************************************************/
/*
 * 'The other ones'
 */

/* simple routine to get the variables ordered in a static manner */
function organize_objects($obj_list, $obj_type) {
	$props = get_object_properties($obj_type);
	$var_index = $props['var_index'];
	foreach($obj_list as $obj_k => $d_obj) {
		$t_obj = '';
		foreach($var_index as $k => $discard) {
			if(isset($d_obj[$k])) $t_obj[$k] = $d_obj[$k];
		}

		$obj_list[$obj_k] = $t_obj;
	}

	return($obj_list);
}

# these two should be replaced with something fancier later
function deny_access($msg) {
	global $page;
	if ($page)
		$page->html_start(); // noop if it's already been called, so harmless
	echo '<img src="images/icons/shield-critical.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
	echo "<h2 style=\"margin-top: 4px;float: left\">Access denied!</h2><p style=\"clear:both\">$msg</p>\n";
	if ($page)
		$page->html_end();
	else
		exit(1);
}

function get_rights() {
	global $webconfig_user;

	get_user(); /* Just to make sure $webconfig_user is set */

	/* Fetch rights from user object. */
	$nacoma_rights = array();
	foreach ($webconfig_user->auth_data as $key => $value) {
		$nacoma_rights[$key] = $value;
	}
	return $nacoma_rights;
}

function is_super_user() {
	$rights = get_rights();
	if ($rights === true)
		return true;

	if (isset($rights['system_commands']) && isset($rights['configuration_information']))
		return true;
	
	return false;
}

/* prints a warning and forces the user to go back or browse help */
function input_missing_error($req, $obj_type) {
	echo "<b>Bad input.</b> A vital keyword is
		   missing from the $obj_type object.<br />\n";
	echo "If you wish to delete an object, you should use the 'delete' button.
		   <br /><br />\n";
	echo "The missing keyword is <b><a href='docs/help.php#$req'>$req</b></a>\n";
	html_end();
}

/* validates a user. Exits with an errormessage if not auth'ed ok.
 * This function also handles connecting to the database and
 * populating the index table, in case user checked out OK */
function validate_user() {
	global $Config;

	/* set up the playground. 'tedious' tasks end up here. */
	if(isset($_REQUEST['action']) && is_string($_REQUEST['action'])) {
		$_REQUEST['action'] = trim($_REQUEST['action']);
	}

	$user = get_user();

	/* superuser(s) can do anything. */
	if(is_super_user())
		return true;

	/* regular user. check for access rights. */
	if( array_key_exists( "configuration_information", get_rights() ) ) {
		return true;
	}
	
	# unauthorized...
	return false;
}

/* sets up the playground. All pages get the same generic look. */
function html_start() {
	global $page, $DEBUG;
	$page->html_start();
	if ($DEBUG)
	  bug("OLD html_start() USED");
}

function html_end($exit_code=0) {
	global $page, $DEBUG;
	if ($DEBUG)
	  bug("OLD html_end() USED");
	$page->html_end($exit_code);
}

/**
 * Tell the user he/she has hit a bug and, optionally, bail out
 * @param $msg The message printed to user
 * @param $crash Print debug backtrace and exit or not?
 * @param $debug_msg Message to, hopefully, include in bug-reports
 */
function bug($msg, $crash = false, $debug_msg = false, $backtrace=false) {
	global $SUPPORT;
	global $DEBUG;

	if (!$backtrace)
		$backtrace = debug_backtrace();

	$num_args = func_num_args();
	echo "<br />A bug was encountered in $_SERVER[PHP_SELF]<br />\n";
	echo "Please contact $SUPPORT regarding this matter.<br /><br /><b>$msg</b><br /><br />\n";

	if($DEBUG && $debug_msg) echo "$debug_msg<br /><br />\n";

	if($crash) {
		if ($DEBUG) {
			echo "Backtrace follows<pre>\n";
			print_r($backtrace);
			echo "</pre>\n";
		}
		else {
			@mkdir('/tmp/nacoma-stacktraces/', 0700, true);
			$file = tempnam('/tmp/nacoma-stacktraces', date('Ymd-hi').'-');
			$fd = fopen($file, 'w');
			fwrite($fd, $msg."\n");
			fwrite($fd, $debug_msg."\n");
			fwrite($fd, var_export($backtrace, true));

			fclose($fd);

			echo '<p>Debug information has been saved to "'.$file.'".</p>';
		}
		html_end(1);
	}
}

/**
 * Checks if this is a poller system 
 * in a distributed setup or not. 
 *
 * @return boolean
 */
function is_poller() {
	$mon_cmd=exec_cmd("(/usr/bin/mon node show --type=master|/bin/grep -c 'TYPE=master') 2> /dev/null");
	return $mon_cmd['buf'] > 0;
}

/**
 * Custom parse_ini_file since default do not handle strings with whitespaces or '='
 *
 * Found in metadata.php
 */
function my_parse_ini_file($metafile){
	$handle = fopen($metafile, 'r');
	$result_array = Array();
	$section = "";
	$plugin = basename(preg_replace('/.metadata/',"",$metafile));
	$result_array['plugin']=$plugin;
	if ($handle) {
		$i=0;
		while (!feof($handle)) {
			$i++;
			$line = fgets($handle);
			/* Strip comments (lines starting with # or ; */
			if(preg_match("/^[#;]/", $line)){
				continue;
			}
			/* Strip blank lines */
			if(trim($line) === ""){
				continue;
			}
			/* Look for section header, enclosed in [] */
			if(preg_match("/^[\[][a-zA-Z0-9]*[\]][ \t]*$/", $line)){
				$section = trim($line, " \t[]\n");
				continue;
			}
			/* This must be a 'section member', store in array  */
			$cmd_name = strtok($line, "=");
			$cmd_line = strtok("");
			if($cmd_line != "")
			  $result_array[$section][$cmd_name] = trim($cmd_line);
			else
			  $result_array[$section][] = trim($cmd_name);
		}
		fclose($handle);
	}
	return($result_array);
}
