
/***************************************
 * User lookup
 **************************************/

function clear_highlights() {
	$('tr').removeClass('highlight_row');
	$('.backendmarker').hide();
}

function set_highlight_group( backend, group ) {
	rows = $('input[type="text"]').filter( function(index){ return this.value == group; } ).parents('tr');
	rows.addClass('highlight_row');
	rows.find('.backendmarker_'+backend).show();
	
	return rows.length > 0;
}

function do_lookup_user( user )
{
	$('#highlight_user_loading').show();
	clear_highlights();
	$('#highlight_user_groups').html( '' );
	$.get( 'groups_per_user_json.php', { username: user }, function(data) {
		$('#highlight_user_loading').hide();
		if( data.error ) {
			alert( data.error );
		} else {
			var missing_groups = new Array();
			$.each( data.groups, function( backend, val ) {
				$.each( val, function( i, grp ) {
					if( !set_highlight_group( backend, grp ) ) {
						missing_groups.push(grp);
					}
				} );
			});
			var missing_groups_html = '<h3>'+user+' is also member of:</h3><ul><li>'+missing_groups.join("</li><li>")+'</li></ul>';
			if(!missing_groups.length) {
				missing_groups_html = "<h3>No extra groups found for "+user+"</h3>";
			}
			set_highlight_group( 'no_specific_backend', 'user_'+user );
			if (user.length > 0) {
				$('#highlight_user_groups').html( missing_groups_html );
			}
		}
	}, "json" );
}

/***************************************
 * Group filter
 **************************************/

function filter_groups( group_filter ) {
	var group = "";
	var reg = new RegExp(group_filter);
	$('.group_field').each( function() {
		group = $(this).val();
		if (!group.match(reg)) {
			$(this).closest('tr').hide();
		} else {
			$(this).closest('tr').show();
		}
	});
}

/***************************************
 * Hover highlight and box sizes
 **************************************/
function helpClass() {
	var that = this;
	$.getJSON(
		'js/grouprights_helptexts.json',
		function (data) {
			that.helptexts = data;
		}
	);
}
grouprightsHelper = new helpClass();
$(document).ready(function() {

	$('#filter_groups_form').submit( function() {
		filter_groups( $('#group_filter_box').val() );
		return false;
	});

	$('#user_lookup_form').submit( function() {
		do_lookup_user( $('#highlight_user').val() );
		return false;
	} );

	if (!$.browser.msie) {
		$('.access_header').each(function() {
			var w = 0;
			var tr = $(this);
			tr.find('span').each(function() {
				v = $(this).width();
				if (w < v) {
					w = v;
				}
			});
			w = Math.sqrt((w*w)/2);
			tr.find('td').height(Math.round(w+0.5));
		});
	}
	var qtip_options = {
		position: {
			my: 'right top',
			at: 'bottom left',
			target: 'event',
			//Confine tooltip to window
			viewport: $(window)
		},
		show: {
			//Show after one second
			delay: 400,
			solo: true
		},
		style: {
			classes: 'ui-tooltip-dark ui-tooltip-rounded'
		}
	};
	var access_header_options = $.extend(
		true,
		{
			content: {
				title: {
					text: function() {
						return $(this).find('span').text();
					}
				},
				text: function() {
					var lcheading = $(this).find('span').text().toLowerCase();
					if (!grouprightsHelper.helptexts[lcheading])
						return 'No help text available';
					return grouprightsHelper.helptexts[lcheading];
				}
			},
			position: {
				at: 'top left'
			}
		},
		qtip_options
	);
	var auth_driver_options = $.extend(
		true,
		{
			content: {
				title: {
					text: function() {
						return "Auth Driver";
					}
				},
				text: function() {
					return "These are the different login methods you have specified. The red and green squares next to the group names signals which group belongs to which authentication method. If you perform a lookup user an X will appear in the groups the user belongs to.";
				}
			}
		},
		qtip_options
	);
	var checkbox_options = $.extend(
		true,
		{
			content: {
				title: {
					text: function() {
						var col = $(this).parent().children().index(this)+1;
						//Get and return text from column header
						return $('.headerGroupRights td:nth-child('+col+') span').text();
					}
				},
				text: function() {
					var html, helpContent;
					var col = $(this).parent().children().index(this)+1;
					var heading = $('.headerGroupRights td:nth-child('+col+') span').text();
					//Get groupname from input box at start of row
					var groupName = $(this).siblings(':first-child').find('input[type="text"]').val();
					//Check if helptext objects were loaded
					var lcheading = heading.toLowerCase();
					if (!grouprightsHelper.helptexts[lcheading])
						helpContent = 'No help text available';
					else
						helpContent = grouprightsHelper.helptexts[lcheading];
					//Format and return tooltip body
					html = '<b>Group:</b> '+groupName+
							'<br />'+helpContent;
					return html;
				}
			}
		},
		qtip_options
	);
	var group_help_icon_options = $.extend(
		true,
		{
			content: {
				title: {
					text: function() {
						return $(this).closest('h2').text();
					}
				},
				text: function() {
					var heading = $(this).attr('title');
					if (!grouprightsHelper.helptexts[heading])
						return 'No help text available';
					return grouprightsHelper.helptexts[heading];
				}
			}
		},
		qtip_options
	);
	var nagvis_checkbox_options = $.extend(
		true,
		{
			content: {
				title: {
					text: function() {
						var col = $(this).parent().children().index(this);
						var mode = col%2 === 0 ? ' edit' : ' view';
						col = Math.round((col)/2)+1;
						//Get and return text from column header
						return $('.nagvis_map_header td:nth-child('+col+') span').text() + mode;
					}
				},
				text: function() {
					var html, helpContent;
					var col = $(this).parent().children().index(this);
					var mode = col%2 === 0 ? ' edit' : ' view';
					col = Math.round((col)/2)+1;
					var heading = $('.nagvis_map_header td:nth-child('+col+') span').text();
					//Get groupname from input box at start of row
					var groupName = $(this).siblings(':first-child').text();
					helpContent = 'Set' + mode + ' rights on ' + heading + ' for group ' + groupName;
					//Format and return tooltip body
					html = '<b>Group:</b> '+groupName+
							'<br />'+helpContent;
					return html;
				}
			}
		},
		qtip_options
	);
	$('.group_help_icon')
		.qtip(group_help_icon_options);
	$('.access_header td').not('.auth_driver').not('.no_tooltip')
		.qtip(access_header_options);
	$('.auth_driver')
		.qtip(auth_driver_options);
	//Returns background colors for hovered columns and rows
	$('.checkbox_cell')
		.hover(
			function() {
				var col = $(this).parent().children().index(this)+1;
				$('.headerGroupRights td:nth-child('+col+') span').css('backgroundColor', '#00ff00');
				$(this).siblings(':first-child').css('backgroundColor', '#00ff00');
			},
			function() {
				var col = $(this).parent().children().index(this)+1;
				$('.headerGroupRights td:nth-child('+col+') span').css('backgroundColor', '');
				$(this).siblings(':first-child').css('backgroundColor', '');
			}
		)
		.qtip(checkbox_options);
	$('.nagvis_checkbox_cell')
		.hover(
			function() {
				$('.group_name').css('backgroundColor', '');
				$('.nagvis_permission_header span').css('backgroundColor', '');
				var col = $(this).parent().children().index(this)+1;
				$('.nagvis_permission_header td:nth-child('+col+') span').css('backgroundColor', '#00ff00');
				$(this).siblings(':first-child').css('backgroundColor', '#00ff00');
			},
			function() {
				$('.nagvis_permission_header span').css('backgroundColor', '');
				var col = $(this).parent().children().index(this)+1;
				$('.nagvis_permission_header td:nth-child('+col+') span').css('backgroundColor', '');
				$(this).siblings(':first-child').css('backgroundColor', '');
			}
		)
		.qtip(nagvis_checkbox_options);

	$('#category_expand')
		.on('click', '.expand_button', function(event, all) {
			var button = $(this);
			// Get class name from buttons id.
			var id = $('.' + button.attr('id'));
			// Only one button is pushed. Hide the clicked category
			if (all === false || all === undefined) {
				// Columns are shown, hide them.
				if (button.attr('value') == '-') {
					id.attr('style', 'display: none;');
					button.attr('value', '+');
				// Columns are hidden, show them.
				} else if (button.attr('value') == '+'){
					id.attr('style', 'display: table-cell;');
					button.attr('value', '-');
				}
			// Expand all button is clicked. Show all categories.
			} else if (all === 'expandAll') {
				id.attr('style', 'display: table-cell;');
				button.attr('value', '-');
			// Contract all button is clicked. Hide all categories.
			} else if (all === 'contractAll') {
				id.attr('style', 'display: none;');
				button.attr('value', '+');
			}
		})
		.on('click', '#expand_all_button', function() {
			$('.expand_button').trigger('click', 'expandAll');
		})
		.on('click', '#contract_all_button', function() {
			$('.expand_button').trigger('click', 'contractAll');
		});
	// Loading of page. All buttons are triggered. Hack to contract all categories and still get correct height on all headers.
	$('.expand_button').trigger('click', false);
});
